package guiAgent;

import java.util.List;
import java.util.Random;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

public class Agent implements player.gamer.statemachine.sadna.FourInARow, GUIPlayer {

	final int nCols = 8;
	final int nRows = 6;
	
	private final Lock lock = new ReentrantLock();
	private final Condition clickedEvent = lock.newCondition();
	private int move;

	private GUI gui;
	private boolean myTurn = false;
	private boolean inGame = false;
	
	private Random generator = new Random();

	@Override
	public void fourStart(String role) {
		System.out.println("You're playing " + role);
		myTurn = role.equals("red");
		inGame = true;
		gui.start(role.equals("red"));
	}
	
	@Override
	public int fourMove(List<Integer> moved, List<Integer> options, long timelimit) {
		timelimit -= System.currentTimeMillis();
		//System.out.println("Opponents "+ moved.toString() + "  Alternatives "+ options.toString() + " Moving " + options.get(1) + " ...had " + timelimit/1000.0 + " seconds");
		
		move = options.get(generator.nextInt(options.size()));
		if (moved.size() > 0)
			gui.move(moved.get(0), true);
		myTurn = true;
		
		lock.lock();
		try {
			clickedEvent.await(timelimit, TimeUnit.MILLISECONDS);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		lock.unlock();

		myTurn = false;
		gui.move(move, false);

		System.out.println("Opponents "+ moved.toString() + "  Alternatives "+ options.toString() + " Moving " + move + " ...had " + timelimit/1000.0 + " seconds");
		// the enclosing class expects the index of the move in the options list, not the column number
		return options.indexOf(move);
	}
	
	@Override
	public void fourEnd(final int score) {
		System.out.println("Score: "+ score);
		inGame = false;
		gui.finish(score);
	}
	
	@Override
	public String getName() {
		return "MeravLior";
	}
	
	public static void main(String[] args) {
		new Agent();
	}
	
	public Agent() {
		gui = new GUI(Agent.this);
	}

	@Override
	public
	void setMove(int move) {
		this.move = move;
		
		lock.lock();
		clickedEvent.signal();
		lock.unlock();
	}

	@Override
	public
	boolean isMyTurn() {
		return myTurn;
	}

	@Override
	public
	boolean isInGame() {
		return inGame;
	}

	@Override
	public int getNRows() {
		return nRows;
	}

	@Override
	public int getNCols() {
		return nCols;
	}

}