package engine;

import board.Board;


public abstract class ExploitExplore {

	protected MiniMax miniMax;
	private int minDepth, maxDepth;
	
	public ExploitExplore(int minDepth, int maxDepth) {
		this.minDepth = minDepth;
		this.maxDepth = maxDepth;
		miniMax = new MiniMax(maxDepth);
	}
	
	/**
	 * 
	 * @param board
	 * @param weights
	 * @param trust
	 * @return the move (i.e. column index) we force, and if nothing is to be forced, then -1
	 */
	public int getNextMove(Board board, double[] weights, double trust) {

		// BOOTSTRAP for moves that we take forcefully
		
		// check for an immediate win
		for(int c=0; c<Constants.nCols; ++c) {
			if (board.isThreat(false, c, board.getTop(c))) {
				return c;
			}
		}

		// then check for an immediate loss
		for(int c=0; c<Constants.nCols; ++c) {
			if (board.isThreat(true, c, board.getTop(c))) {
				return c;
			}
		}

		setMaxDepth((int)Math.floor(minDepth + (maxDepth-minDepth)*trust));
	
		return -1;
	}
		
	/**
	 * 
	 * @param maxDepth - maximun tree depth
	 */	
	void setMaxDepth(int maxDepth) {
		miniMax.setMaxDepth(maxDepth);
	}
	
}
